/**************************************************************************************\
 * Source code for Mpong.                                                             *
 * Ken Nelson, Jonathan Schmoll                                                       *
 * 12-13-99                                                                           *
\**************************************************************************************/

#define WIN32_LEAN_AND_MEAN
#define INITGUID

#include <windows.h>
#include <mmsystem.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>

#include <ddraw.h>
#include <dinput.h>
#include "ddutil.h"
#include "Mpong.h"

void DDraw_Init(void);						// Initializes DirectDraw
void DInput_Init(void);						// Initializes DirectInput
void Error(char*);							// Handles error messages.

void Reset_Positions(void);					// Defaults the positions of surfaces.
void Game_Init(void);						// Loads SPRITEs with images.
void Game_Main(void);						// Game loop, controls game overhead.
void Main_State(int&, int&);				// Game logic loop.
void Game_End(void);						// Called when game over.


SPRITE_PTR background = new SPRITE;			// Background SPRITE.
SPRITE_PTR ball = new SPRITE;				// Ball SPRITE.
SPRITE_PTR paddle = new SPRITE;				// Paddle SPRITE.

TEXTMESSAGE_PTR score = new TEXTMESSAGE;	// Score text message.
TEXTMESSAGE_PTR lives = new TEXTMESSAGE;	// Number of lives text message.
TEXTMESSAGE_PTR level = new TEXTMESSAGE;	// Current level text message.
TEXTMESSAGE_PTR start = new TEXTMESSAGE;	// Text message seen at start.


// This handles all windows messages.
LRESULT CALLBACK WindowProc(HWND hwnd, 
							UINT msg, 
							WPARAM wparam, 
							LPARAM lparam) {


	PAINTSTRUCT ps;							// used to hold the paint info

	switch(msg) { 
	
		// This message occurs when your window needs repainting
		case WM_PAINT: {
		
			hdc = BeginPaint(hwnd,&ps); 
			EndPaint(hwnd,&ps);

			return(0);
		} break;

		// This message occurs when a key is pressed.
		case WM_KEYDOWN: {
			
			// If the key is ESCAPE, kill the application.
			if((int)wparam == VK_ESCAPE)
				PostQuitMessage(0);

			return(0);
		} break;

		// This message is sent when your window is destroyed
		case WM_DESTROY: {
		
			PostQuitMessage(0);
			return(0);
		} break;

		default:break;
	} 

	return (DefWindowProc(hwnd, msg, wparam, lparam));
} 


// This actually creates the windows and contains the main loop.
int WINAPI WinMain( HINSTANCE hinstance,
                    HINSTANCE hprevinstance,
                    LPSTR lpcmdline,
                    int ncmdshow) {


	MSG msg;								// This holds a generic message


	// Specifies information about the window.
	winclass.cbSize = sizeof(WNDCLASSEX);
	winclass.style = CS_DBLCLKS | 
	                 CS_OWNDC | 
	   				 CS_HREDRAW | 
	 				 CS_VREDRAW;

	winclass.lpfnWndProc = WindowProc;
	winclass.cbClsExtra = 0;
	winclass.cbWndExtra = 0;
	winclass.hInstance = hinstance;
	winclass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	winclass.hIconSm = LoadIcon(NULL, IDI_APPLICATION);
	winclass.hCursor = LoadCursor(NULL, IDC_ARROW);
	winclass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	winclass.lpszMenuName = NULL;
	winclass.lpszClassName = "Example Window";


	// Registers window.
	if (!RegisterClassEx(&winclass))
		return(0);

	// Creates the Window.
	if (!(hwnd = CreateWindowEx(WS_EX_TOPMOST,
								"Example Window",					// Class
								"DirectDraw Example",				// Title
								WS_VISIBLE | WS_OVERLAPPEDWINDOW,
								0,0,								// Starting coordinates.
								SCREEN_WIDTH,						// Size of
								SCREEN_HEIGHT,						// Window.
								NULL,								
								NULL, 
								hinstance, 
								NULL))) 
		return(0);


	ShowCursor(FALSE);								// Hide the mouse cursor.
	DDraw_Init();									// Initialize DirectDraw.
	DInput_Init();									// Initialize DirectInput.
	Game_Init();									// Initialize SPRITEs.

	// Main windows loop continues until Quit message is sent.
	while(1) {
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE)) { 
	
			if (msg.message == WM_QUIT)
				break;

			TranslateMessage(&msg);
			DispatchMessage(&msg);
		} 
		else {
			Game_Main();			// Game loop.
		} 
	} 

	return(msg.wParam);

} 


// Initializes DirectDraw.
void DDraw_Init() {

	HRESULT ddrval;				// Result used for testing.
	DDSCAPS ddscaps;			// Surface Capabilities.


	// Create the DirectDraw object.
	ddrval = DirectDrawCreate( NULL, &lpdd, NULL );
	if( ddrval != DD_OK )
		Error("DirectDrawCreate Failed!");

	// Specify how to share resources with OS.
	ddrval = lpdd->SetCooperativeLevel( hwnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN );
	if( ddrval != DD_OK)
		Error("SetCooperativeLevel Failed!");

	// Set the screen resolution to SCREEN_WIDTH X SCREEN_HEIGHT
	ddrval = lpdd->SetDisplayMode(SCREEN_WIDTH, SCREEN_HEIGHT, 16);
	if( ddrval != DD_OK)
		Error("SetDisplayMode Failed!");
 

	// Specify the primary surface to draw on. (screen)
	memset(&ddsd, 0, sizeof(ddsd)); 
	ddsd.dwSize = sizeof( ddsd );
	ddsd.dwFlags = DDSD_CAPS | 
		           DDSD_BACKBUFFERCOUNT;

	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | 
		                  DDSCAPS_FLIP | 
						  DDSCAPS_COMPLEX;

	ddsd.dwBackBufferCount = 1;

	// Create the primary surface.
	ddrval = lpdd->CreateSurface( &ddsd, &lpddsPrimary, NULL );
	if( ddrval != DD_OK )
		Error("CreateSurface Failed!");

	ddscaps.dwCaps = DDSCAPS_BACKBUFFER;

	// Attach an offscreen drawing surface.
	ddrval = lpddsPrimary->GetAttachedSurface(&ddscaps, &lpddsBack);
	if( ddrval != DD_OK )
		Error("GetAttached Surface Failed!");

}


// Initializes DirectInput.
void DInput_Init() {

	HRESULT ddrval;			// Result used for testing.


	// Create the DirectInput object.
	ddrval = DirectInputCreate(winclass.hInstance, DIRECTINPUT_VERSION, &lpdi, NULL);
	if(ddrval != DI_OK)
		Error("DirectInputCreate Failed!");

	// Create the DirectInput device (mouse).
	ddrval = lpdi->CreateDevice(GUID_SysMouse, &lpdiMouse, NULL);
	if(ddrval != DI_OK) 
		Error("CreateDevice DINPUT Failed!");

	// Specify how to share the ouse with OS.
	ddrval = lpdiMouse->SetCooperativeLevel(hwnd, DISCL_BACKGROUND |
												  DISCL_NONEXCLUSIVE);
	if(ddrval != DI_OK) 
		Error("SetCooperativeLevel DINPUT Failed!");

	// Specify how to interpret mouse feedback.
	ddrval = lpdiMouse->SetDataFormat(&c_dfDIMouse);
	if(ddrval != DI_OK)
		Error("SetDataFormat DINPUT Failed!");

	// Finally acquire the mouse.
	ddrval = lpdiMouse->Acquire();
	if(ddrval != DI_OK)
		Error("Acquire DINPUT Failed!");
}


// Initializes the game... resets positions and loads bitmaps into sprites.
void Game_Init(void) {

	// Initializes default positions.
	Reset_Positions();

	// Loads bitmaps into each sprite.
	background->Load_Sprite("images/arena2.bmp", 800, 600);
	ball->Load_Sprite("images/example2.bmp", 88, 104);
	paddle->Load_Sprite("images/paddle4.bmp", 144, 32);

}


// Game overhead loop.  Controls start of the game or end of the game, etc..
void Game_Main(void) {

	static int game_running = 0;		// Is the game running? Default to no.
	static int lives = 2;				// How many lives? 3 to start.
	static int new_game = 1;			// Is this a new game? Why yes it is.

	// This is a new game.
	if(!game_running && new_game) {

		// Get information from the mouse.
		lpdiMouse->GetDeviceState(sizeof(DIMOUSESTATE), (LPVOID)&mouse_state); //test later

		// Draw the sprites to the off screen surface.
		background->Draw_Sprite(lpddsBack);
		ball->Draw_Sprite(lpddsBack);
		paddle->Draw_Sprite(lpddsBack);

		start->Set_Message("Click to Start | Hit ESCAPE to end.", NULL);
		start->Write_Message();

		// Flip the offscreen surface to be on screen.
		while(lpddsPrimary->Flip(NULL, DDFLIP_WAIT) != DD_OK);	
	}

	// New game and the mouse was clicked.
	if(mouse_state.rgbButtons[0] && new_game) {

		// Play opening sound.
		PlaySound("sounds/destroy.wav", NULL, SND_FILENAME | SND_SYNC);

		// Game is now running.
		game_running = 1;

		// Its no longer a new game.
		new_game = 0;
	}


	// Died but still playing.
	if(lives >= 0 && !new_game && !game_running) {

		// The game is still running.
		game_running = 1;

		// Reset the sprite positions on death.
		Reset_Positions();
	}

	// Game still running. Called each cycle if game is active.
	if(game_running && lives >= 0)
		// Game logic loop called.
		Main_State(game_running, lives);

	// Game Over
	if(lives < 0) {
		// End the game when all lives are lost.
		Game_End();
	}
}


// Game logic loop.  Controls ball, and paddle.
void Main_State(int &game_running, int &num_lives) {

	static int mouse_x = SCREEN_WIDTH/2;	// Initialize mouse position.
	static int score_count = 0;				// Keeps the players score.
	static int num_level = 1;				// Players current level.
	static int factor = 3;					// Velocity factor.

	// Retrieve current position of the paddle.
	RECT paddle_dest = paddle->Get_Dest();

	// Retrieve current position of ball.
	RECT ball_dest = ball->Get_Dest();

	// Retrieve current x and y velocities for ball.
	int vel_x = ball->Get_VelX();
	int vel_y = ball->Get_VelY();

	// Retrieve information from the mouse.
	lpdiMouse->GetDeviceState(sizeof(DIMOUSESTATE), (LPVOID)&mouse_state); 
	mouse_x = mouse_state.lX;


	// Controls movement for paddle.

	// If the mouse moved left.
	if(mouse_x < 0) {

		// Is the paddle going to fall off the screen?
		// If so, set it to a constant.
		if(paddle_dest.left + mouse_x < 28) 
			paddle_dest.left = 28;

		// If not, change the paddle's position by how much the mouse moved.
		else 
			paddle_dest.left += mouse_state.lX;
	}

	// If the mouse moved right.
	else {

		// Is the paddle going to fall off the screen?
		// If so, set it to a constant.
		if(paddle_dest.right + mouse_x > SCREEN_WIDTH - 28) 
			paddle_dest.left = SCREEN_WIDTH - paddle->Get_Width() - 28;

		// If not, change the paddle's position by how much the mosue moved.
		else 
			paddle_dest.left += mouse_state.lX;
	}

	// Update the right edge of the paddle in relation to the new left edge.
	paddle_dest.right = paddle_dest.left + paddle->Get_Width();


	// Controls Movement for ball.

	// Is ball going to fall off the left edge of the screen?
	// If so, make the x velocity positive.
	if(ball_dest.left + vel_x < 28) {
		ball->Set_VelX((rand()%7) + factor);
		PlaySound("sounds/click.wav", NULL, SND_FILENAME | SND_ASYNC);
	}

	// Is ball going to fall off the right edge of the screen?
	// If so, make the x velocity negative.
	else if(ball_dest.right + vel_x > SCREEN_WIDTH - 28) {
		ball->Set_VelX( -((rand()%7) + factor));
		PlaySound("sounds/click.wav", NULL, SND_FILENAME | SND_ASYNC);
	}

	// Is ball going to fall off the top edge of the screen?
	// If so, change the y velocity to positive.
	if(ball_dest.top + vel_y < 28) {
		ball->Set_VelY((rand()%7) + factor);
		PlaySound("sounds/click.wav", NULL, SND_FILENAME | SND_ASYNC);
	}

	// Is ball going to fall off the bottom edge of the screen?
	else if(ball_dest.bottom + vel_y > SCREEN_HEIGHT) {
	
		// If this is not the player's last life, play this sound.
		if(num_lives > 0)
			PlaySound("sounds/powers.wav", NULL, SND_FILENAME | SND_SYNC);

		// If this is the player's last life, play a different sound.
		else
			PlaySound("sounds/failedme.wav", NULL, SND_FILENAME | SND_SYNC);

		// The player lost so the game is not running.
		game_running = 0;

		// Player's lives gets decreased.
		num_lives--;
	}


	// For ball to bounce off of paddle.

	// Is ball touching the paddle?
	// If so, make him bounce.
	if(ball_dest.right >= paddle_dest.left && 
		ball_dest.left <= paddle_dest.right) {

		if(ball_dest.bottom >= paddle_dest.top - 3 &&
			ball_dest.bottom <= paddle_dest.bottom + 3) {

			// Change the y velocity to negative.
			ball->Set_VelY( -((rand()%7) + factor));

			// Play a sound.
			PlaySound("sounds/click.wav", NULL, SND_FILENAME | SND_ASYNC);

			// Increase the score.
			score_count++;
		}
	}


	// Update ball's position.
	ball_dest.left += vel_x;
	ball_dest.top += vel_y;
	ball_dest.right += vel_x;
	ball_dest.bottom += vel_y;

	// Set the new destinations for ball and for the paddle.
	ball->Set_Position(ball_dest);
	paddle->Set_Position(paddle_dest);

	// Draw all the sprites to the off screen surface.
	background->Draw_Sprite(lpddsBack);
	ball->Draw_Sprite(lpddsBack);
	paddle->Draw_Sprite(lpddsBack);


	// Display Score, level, and lives.

	// If the score is more than zero, display it.
	if(score_count > 0) {
		score->Set_Message("SCORE: %d", score_count*100);
		score->Write_Message();
	}

	// Otherwise, dont.
	else {
		score->Set_Message("SCORE: ", NULL);
		score->Write_Message();
	}

	// If the player has a life to spare, display it.
	if(num_lives > 0) {
		lives->Set_Message("LIVES: %d", num_lives);
		lives->Write_Message();
	}

	// Otherwise dont.
	else {
		lives->Set_Message("LIVES: ", NULL);
		lives->Write_Message();
	}

	// Display the player's level
	level->Set_Message("LEVEL: %d", num_level);
	level->Write_Message();

	// If the players score is high enough, increase the level.
	if(score_count*100 >= num_level*5000) {

		// Level goes up.
		num_level++;

		// Difficulty goes up.
		factor++;
	}

	// Flip the off screen surface to be on screen.
	while(lpddsPrimary->Flip(NULL, DDFLIP_WAIT) != DD_OK);	

}

// Resets the sprites positions.
void Reset_Positions() {

	RECT temp;

	// Set ball's default position.
	temp.left   = SCREEN_WIDTH/2 - 44;
	temp.top    = SCREEN_HEIGHT/2 - 52;
	temp.right  = temp.left + 88;
	temp.bottom = temp.top + 104;
	ball->Set_Position(temp);

	// Set ball's default x and y velocities.
	ball->Set_VelX((rand()%7) + 3);
	ball->Set_VelY( -((rand()%7) + 3));


/////////////////////////////////////////////////////////////////////

	// Set the paddle's default position.
	temp.left   = SCREEN_WIDTH/2 - 68;
	temp.top    = SCREEN_HEIGHT/2 + 252;
	temp.right  = temp.left + 140;
	temp.bottom = temp.top + 24;
	paddle->Set_Position(temp);


/////////////////////////////////////////////////////////////////////

	// Set Lives and Level positions.
	temp.left   = 0;
	temp.top    = 15;
	temp.right  = temp.left + 300;
	temp.bottom = 100;
	
	lives->Set_Dest(temp);

	temp.top    = 30;
	level->Set_Dest(temp);

	// Set Start position
	temp.left   = SCREEN_WIDTH/2 - 120;
	temp.top    = SCREEN_HEIGHT/2 - 300;
	temp.right  = temp.left + 240;
	temp.bottom = temp.top + 600;

	start->Set_Dest(temp);
}


// Game has ended.
void Game_End() {
	PostQuitMessage(0);
}


// Display error messages.
void Error(char* message) {
	MessageBox(hwnd, message, message, MB_OK);
	PostQuitMessage(0);
}