#define SCREEN_WIDTH   1024
#define SCREEN_HEIGHT  768

LPDIRECTDRAW lpdd;					// DirectDraw Object.
LPDIRECTDRAWSURFACE lpddsPrimary;	// DirectDraw Surface (screen).
LPDIRECTDRAWSURFACE lpddsBack;		// DirectDraw Surface (offscreen).
DDSURFACEDESC ddsd;					// Surface Descriptor.

LPDIRECTINPUT lpdi;					// DirectInput Object.
LPDIRECTINPUTDEVICE lpdiMouse;		// DirectInput device.
DIMOUSESTATE mouse_state;			// State of the mouse.

WNDCLASSEX winclass; 
HWND hwnd; 
HDC hdc;							// Handle to device context.

// Sprites are the objects that contain bitmaps and positions for
// drawing on the screen.
typedef class SPRITE {

	public:
		SPRITE(void);

		// Loads the sprite with an image.
		void Load_Sprite(char*, int, int);

		// Draws the sprite to the specified surface.
		void Draw_Sprite(LPDIRECTDRAWSURFACE);

		// Retrieves sprite's current position.
		RECT Get_Dest(void) {return this->dest;}

		// Retrieves sprite's width and height.
		int Get_Width(void) {return this->width;}
		int Get_Height(void){return this->height;}

		// Retrieves sprite's current x and y velocities.
		int Get_VelX(void) {return this->vel_x;}
		int Get_VelY(void){return this->vel_y;}

		// Sets new x and y velocities.
		void Set_VelX(int vel_x) {this->vel_x = vel_x;}
		void Set_VelY(int vel_y){this->vel_y = vel_y;}

		// Sets noew sprite destination.
		void Set_Position(RECT);


	private:
		// Sprite velocities.
		int vel_x, vel_y;

		// Sprite width and height.
		int width, height;

		// Sprite drawing destination.
		RECT dest;

		// Sprite image.
		LPDIRECTDRAWSURFACE image;

} *SPRITE_PTR;


// Constructor, defaults all values on sprite creation.
SPRITE::SPRITE() {
	vel_x = 0;
	vel_y = 0;

	width = 0;
	height = 0;

	dest.left = NULL;
	dest.top = NULL;
	dest.right = NULL;
	dest.bottom = NULL;

}


// Loads the specified bitmap to the sprite's surface.
void SPRITE::Load_Sprite(char* filename, int width, int height) {

	DDCOLORKEY color_key;

	// Allocate memory for sprite.
	memset(&ddsd, 0, sizeof(ddsd)); 
	ddsd.dwSize   = sizeof(ddsd);  
	
	ddsd.dwFlags  = DDSD_CAPS | 
		            DDSD_WIDTH | 
					DDSD_HEIGHT;

	// Set the width and height for the sprite.
	ddsd.dwWidth  = this->width = width;
	ddsd.dwHeight = this->height = height;

	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | 
		                  DDSCAPS_SYSTEMMEMORY;

	// Create the sprites surface.
	lpdd->CreateSurface(&ddsd, &(this->image), NULL); 

	color_key.dwColorSpaceLowValue  = 0;   // Set up the transparent color to be black.
	color_key.dwColorSpaceHighValue = 0;   

	// Attach transparent color to sprites surface.
	this->image->SetColorKey(DDCKEY_SRCBLT, &color_key);

	// Load the surface with the specified image.
	DDReLoadBitmap(this->image, filename); 

}


// Draws the sprites image to the specified surface.
void SPRITE::Draw_Sprite(LPDIRECTDRAWSURFACE temp) {

	// Background image or undefine destination RECT.
	if(this->dest.left == NULL)
		temp->Blt(NULL, this->image, NULL, (DDBLT_WAIT), NULL);

	else
		temp->Blt(&dest, this->image, NULL, (DDBLT_WAIT | DDBLT_KEYSRC), NULL);
}


// Sets new destination.
void SPRITE::Set_Position(RECT temp) {
	this->dest.left = temp.left;
	this->dest.top = temp.top;
	this->dest.right = temp.right;
	this->dest.bottom = temp.bottom;
}


// TEXTMESSAGE Class, for text output to screen.
typedef class TEXTMESSAGE {
	
	public:
		TEXTMESSAGE(void);

		// Sets up the message to be drawn.
		void Set_Message(const char*, int);

		// Sets up the destination to draw it.
		void Set_Dest(RECT dest) {this->dest = dest;}

		// Draws the message to the screen.
		void Write_Message(void);

	private:

		// The actual message.
		char* message;

		// The destination.
		RECT dest;

} *TEXTMESSAGE_PTR;


// Constructor, defaults destination to be the entire screen.
TEXTMESSAGE::TEXTMESSAGE() {
	message = NULL;

	dest.left   = 0;
	dest.top    = 0;
	dest.right  = SCREEN_WIDTH - 1;
	dest.bottom = SCREEN_HEIGHT - 1;
}


// Sets up the message to be output.
void TEXTMESSAGE::Set_Message(const char* message, int number) {

	// Allocate memory for message.
	this->message = new char[sizeof(message) + 1];

	// If simple text output.
	if(number == NULL)
		sprintf(this->message, message);

	// Else if simple text plus a number, append the number.
	else
		sprintf(this->message, message, number);
}


// Writes out the message to the screen.
void TEXTMESSAGE::Write_Message(void) {

	// Captures the device context.
	lpddsBack->GetDC(&hdc);

		// Set the background color and text color.
		SetBkColor(hdc, RGB(0,0,0));
		SetBkMode(hdc, OPAQUE);              
		SetTextColor(hdc, RGB(255,255,255));      

		// Draw the text to the given destination.
		DrawText(hdc, this->message, -1, &(this->dest), NULL);
	
	// Release the device context.
	lpddsBack->ReleaseDC(hdc);

}